package tst

import (
	"encoding/hex"
	"testing"
)

func AssertEqual[T comparable](t *testing.T, actual T, expected T) {
	if actual != expected {
		t.Errorf("values differ: Actual: '%v', Expected: '%v'", actual, expected)
	}
}

func AssertNotEqual[T comparable](t *testing.T, actual T, expected T) {
	if actual == expected {
		t.Errorf("values do not differ: Actual: '%v', Expected: '%v'", actual, expected)
	}
}

func AssertDeRefEqual[T comparable](t *testing.T, actual *T, expected T) {
	if actual == nil {
		t.Errorf("values differ: Actual: NIL, Expected: '%v'", expected)
	}
	if *actual != expected {
		t.Errorf("values differ: Actual: '%v', Expected: '%v'", actual, expected)
	}
}

func AssertPtrEqual[T comparable](t *testing.T, actual *T, expected *T) {
	if actual == nil && expected == nil {
		return
	}
	if actual != nil && expected != nil {
		if *actual != *expected {
			t.Errorf("values differ: Actual: '%v', Expected: '%v'", *actual, *expected)
		} else {
			return
		}
	}
	if actual == nil && expected != nil {
		t.Errorf("values differ: Actual: nil, Expected: not-nil")
	}
	if actual != nil && expected == nil {
		t.Errorf("values differ: Actual: not-nil, Expected: nil")
	}
}

func AssertHexEqual(t *testing.T, expected string, actual []byte) {
	actualStr := hex.EncodeToString(actual)
	if actualStr != expected {
		t.Errorf("values differ: Actual: '%v', Expected: '%v'", actualStr, expected)
	}
}

func AssertTrue(t *testing.T, value bool) {
	if !value {
		t.Error("value should be true")
	}
}

func AssertFalse(t *testing.T, value bool) {
	if value {
		t.Error("value should be false")
	}
}
